module Geld
  class Config

    # Set a new currency configuration.
    # If the rates table is set more than once, the new one overrides the older
    def add_currency(currency, rates)
      validate_currency(currency)
      validate_rates(rates)

      @currencies ||= {}

      # Converts rates to BigDecimal
      bd_rates = rates.inject({}){|hash, (k,v)| hash[k] = BigDecimal.new(v.to_s); hash }
      @currencies[currency] = bd_rates

      bd_rates.each do |a_currency, a_rate|

        # reverse operation will be the first element
        current_rate = BigDecimal.new("1").div(a_rate, 2) # 1/rate 2 digits
        @currencies[a_currency] = { currency => current_rate }

        # now the others
        (bd_rates.keys - [a_currency]).each do |other_currency|
          @currencies[a_currency][other_currency] = @currencies[currency][other_currency].mult(current_rate, 2)
        end

      end
      # returns the new calculations
      @currencies
    end

    # returns the specific rate
    def get_rate(from, to)
      @currencies &&
      @currencies[from] &&
      @currencies[from][to]
    end

    private

    # Validates currency entry
    def validate_currency(currency)
      unless currency.is_a?(String) && !currency.empty?
        raise Error::CurrencyError.new("Invalid currency. Should be a non empty string")
      end
    end

    # Validates rates entry
    def validate_rates(rates)
      unless rates.is_a?(Hash) &&
        !rates.empty? &&
        !rates.keys.any?{|key| !key.is_a?(String) } &&
        !rates.values.any?{|value| !value.is_a?(Numeric) }
        raise Error::CurrencyError.new("Invalid rates format. Should be a not empty hash of { String => Numeric }")
      end
    end

  end
end
